@@ -19,7 +19,7 @@
 #ifndef MARKER_CONTROL_H
 #define MARKER_CONTROL_H
 
-int init_marker_control(void);
+int init_ust_marker_control(void);
 int ltt_probe_register(struct ltt_available_probe *pdata);
 
 #endif /* MARKER_CONTROL_H */
