@@ -29,8 +29,8 @@ int main()
 	printf("Basic test program\n");
 
 	for(i=0; i<50; i++) {
-		trace_mark(ust, bar, "str %s", "FOOBAZ");
-		trace_mark(ust, bar2, "number1 %d number2 %d", 53, 9800);
+		ust_marker(bar, "str %s", "FOOBAZ");
+		ust_marker(bar2, "number1 %d number2 %d", 53, 9800);
 		usleep(100000);
 	}
 
