@@ -1,4 +1,6 @@
-/* Copyright (C) 2009  Pierre-Marc Fournier
+/*
+ * Copyright (C) 2009  Pierre-Marc Fournier
+ * Copyright (C) 2011  Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -15,10 +17,12 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
-#include "tp.h"
 #include <ust/marker.h>
 #include "usterr.h"
 
+#define TRACEPOINT_CREATE_PROBES
+#include "tp.h"
+
 struct hello_trace_struct {
 	char *message;
 };
@@ -27,18 +31,25 @@ struct hello_trace_struct hello_struct = {
 	.message = "ehlo\n",
 };
 
-DEFINE_TRACE(hello_tptest);
-
 void tptest_probe(void *data, int anint)
 {
 	struct hello_trace_struct *hello;
-	hello=(struct hello_trace_struct *)data;
+	hello = (struct hello_trace_struct *)data;
 	DBG("in tracepoint probe...");
 	printf("this is the message: %s\n", hello->message);
 }
 
+void tptest2_probe(void *data)
+{
+}
+
 static void __attribute__((constructor)) init()
 {
 	DBG("connecting tracepoint...\n");
-	register_trace_hello_tptest(tptest_probe, &hello_struct);
+	/*
+	 * Note: this is an internal API that will be used within
+	 * TRACEPOINT_EVENT only eventually.
+	 */
+	__register_tracepoint(hello_tptest, tptest_probe, &hello_struct);
+	__register_tracepoint(hello_tptest2, tptest2_probe, &hello_struct);
 }
