@@ -1,4 +1,6 @@
-/* Copyright (C) 2010 Nils Carlson <nils.carlson@ericsson.com>
+/*
+ * Copyright (C) 2010 Nils Carlson <nils.carlson@ericsson.com>
+ * Copyright (C) 2011 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -14,17 +16,20 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
  */
+
 #include <stdio.h>
 #include <ust/clock.h>
 
-#define CREATE_TRACE_POINTS
+#define TRACEPOINT_CREATE_PROBES
 #include "trace_event_test.h"
 
 int main(int argc, char * argv[])
 {
-	static unsigned long time, i;
-	for (i=0; i<10; i++) {
-		time=trace_clock_read64();
-		trace_test(time, i);
+	unsigned long time, i;
+
+	for (i = 0; i < 10; i++) {
+		time = trace_clock_read64();
+		tracepoint(test, time, i);
 	}
+	return 0;
 }
